@@ -24,20 +24,24 @@
 # PFx Brick file system helpers
 
 import os
+
 from pfxbrick import *
+from pfxbrick.pfxdict import file_attr_dict, fileid_dict
 from pfxbrick.pfxhelpers import *
+from pfxbrick.pfxmsg import cmd_file_dir, usb_transaction
 
+# fmt: off
 try:
-    from rich.progress import (
-        BarColumn,
-        DownloadColumn,
-        Progress,
-        TaskID,
-        TextColumn,
-        TimeRemainingColumn,
-        TransferSpeedColumn,
-    )
+    from rich.progress import (BarColumn, DownloadColumn, Progress, TaskID,
+                               TextColumn, TimeRemainingColumn,
+                               TransferSpeedColumn)
 
+    has_rich = True
+except:
+    has_rich = False
+# fmt: on
+
+if has_rich:
     progress = Progress(
         TextColumn("[white]{task.fields[filename]}", justify="right"),
         BarColumn(bar_width=None),
@@ -49,9 +53,6 @@ try:
         "•",
         TimeRemainingColumn(),
     )
-    has_rich = True
-except:
-    has_rich = False
 
 
 def fs_get_fileid_from_name(hdev, name):
@@ -71,7 +72,7 @@ def fs_get_fileid_from_name(hdev, name):
     return fileid
 
 
-def fs_error_check(res):
+def fs_error_check(res, silent=False):
     """
     Convenience error status lookup function used by other file system functions.
 
@@ -79,7 +80,8 @@ def fs_error_check(res):
     :returns: True if there is an error, False on success
     """
     if res > 62:
-        print("File system error: [%02X] %s" % (res, get_error_str(res)))
+        if not silent:
+            print("File system error: [%02X] %s" % (res, get_error_str(res)))
         return True
     else:
         return False
@@ -101,7 +103,7 @@ def fs_format(hdev, quick=False):
     fs_error_check(res[1])
 
 
-def fs_remove_file(hdev, fid):
+def fs_remove_file(hdev, fid, silent=False):
     """
     Sends an ICD message to remove a file from the PFx Brick file system.
 
@@ -111,7 +113,7 @@ def fs_remove_file(hdev, fid):
     msg = [PFX_CMD_FILE_REMOVE]
     msg.append(fid)
     res = usb_transaction(hdev, msg)
-    fs_error_check(res[1])
+    fs_error_check(res[1], silent=silent)
 
 
 def fs_copy_file_to(hdev, fid, fn, show_progress=True, with_bytes=None):
@@ -440,6 +442,29 @@ class PFxFile:
         )
         return s
 
+    def colour_str(self):
+        """
+        Convenient human readable colour string of a file directory entry.
+        """
+        attr_str = ""
+        if self.attributes in file_attr_dict:
+            attr_str = file_attr_dict[self.attributes]
+        elif self.id in fileid_dict:
+            attr_str = fileid_dict[self.id]
+        s = "%3d %-24s [bold white]%6.1f kB[/bold white] [bold blue]%04X[/bold blue] [bold black]%08X %08X[/bold black] [bold aquamarine3]%08X[/bold aquamarine3]  [bold black]%04X %02X[/bold black] %s" % (
+            self.id,
+            self.name,
+            float(self.size / 1000),
+            self.attributes,
+            self.userData1,
+            self.userData2,
+            self.crc32,
+            self.firstSector,
+            self.id,
+            attr_str,
+        )
+        return s
+
 
 class PFxDir:
     """
@@ -475,6 +500,18 @@ class PFxDir:
                 return f
         return None
 
+    def get_filename(self, fid):
+        """
+        Returns a filename with a numeric file ID
+
+        :param int fid: the unique file ID of desired directory entry
+        :returns: :obj:`str` filename of file ID, or None
+        """
+        f = self.get_file_dir_entry(fid)
+        if f is not None:
+            return f.name
+        return None
+
     def find_available_file_id(self):
         """
         Returns the next available unique file ID from the file system.
@@ -538,3 +575,35 @@ class PFxDir:
         )
         s = "\n".join(sb)
         return s
+
+    def colour_dir(self):
+        """
+        Convenient human readable string of the file directory with colour.
+        """
+        sb = []
+        sb.append(
+            "[bold yellow]%3s %-24s %6s    %4s %8s %8s %8s %5s %s[/bold yellow]"
+            % (
+                "ID",
+                "Name",
+                "Size",
+                "Attr",
+                "User1",
+                "User2",
+                "CRC32",
+                "Start",
+                "Ext Attr",
+            )
+        )
+        for f in self.files:
+            sb.append(f.colour_str())
+        sb.append(
+            "%d files, %.1f kB used, %.1f kB remaining"
+            % (
+                len(self.files),
+                float(self.bytesUsed / 1000),
+                float(self.bytesLeft / 1000),
+            )
+        )
+        s = "\n".join(sb)
+        return s
